;rotozoomer
;precalced version

;for every line of your screen you:
;1. take your start u and stepux into your blitter srcstart and blitter srcstep and blit into the lo-byte of your blitter copy pixels setup
;2. take your start v and stepvx into your blitter srcstart and blitter srcstep and blit into the hi-byte of your blitter copy pixels setup
;3. start the 160 chained blits to copy 160 pixels to your screen for that line
;4. so if start is 1 and step is 2, it will blit: 1, 3, 5, 7, ...

;$78000 = vram (visible)
;$70000 = working buff (render buffer)
;$10000+$54000 = $64000 length of 128x128x21 blit list

;                       VBXE summary
;
; $D640   video control bit0 - xdl enable
;                       bit1 - enable 256 colors for GTIA instead of 128
;                       bit2 - disable transparency for color 0
;                       bit3 - enable color $xF being transparent
; $D641-3 XDL address low/mid/high
; $D644   MSEL write $Cx to update palette x (after MB0), write $80 to
; $D645   MB0 write colr index (after RGB) \               update 
; $D646   MB1 write R component             \             priority
; $D647   MB2 write G component              \             definitions
; $D648   MB3 write B component               \
; $D649   colmask - mask applied to overlay color for collision detection
; $D64A   colclr - resets coldetect
;       read = coldetect - bits correspond to GTIA colors that have collided
;                          with an overlay color
; $D64C   ma-cpu   \ bit 7 enables VRAM window at $2000-$3FFF
; $D64D   ma-antic / bits 0-5 select bank
; $D64E   mb-cpu   \ bit 7 enables VRAM window at $4000-$7FFF
; $D64F   mb-antic / bits 0-4 select bank
; $D650-2 blitter address
;       read $D650 = collision code >0 pixel value that was overwritten
; $D653   blitter start - write 1 to start, write 0 to stop
;       read = blitter busy (busy when non-zero)
; $D654   IRQ control - bit 0 enables blitter done IRQ
;
; XDL 
; byte 0
;   bit 0 - set to enable text mode
;   bit 1 - set to enable gfx mode
;   bit 2 - set to disable overlay
;   bit 3 - set to enable color attributes
;   bit 4 - set to disable color attributes
;   bit 5 - RPTL
;   bit 6 - OVADR
;   bit 7 - OVSCRL
; byte 1
;   bit 0 - CHBASE
;   bit 1 - MAPADR (color attributes)
;   bit 2 - MAPPAR (color attributes)
;   bit 3 - OVATT
;   bit 4 - enable hi-res mode (640)
;   bit 5 - enable low-res mode (160)
;   bit 6 - reserved
;   bit 7 - end of XDL, wait for VBLANK

; RPTL - number of scanlines-1 for this modeline
; OVADR - 3 bytes overlay address, 2 bytes line step size
; OVSCRL - 1 byte hscroll, 1 byte vscroll
; CHBASE - high byte of charset address
; MAPADR - 3 bytes map address, 2 bytes line step size
; MAPPAR - 1b hscroll, 1b vscroll, 1b cell width-1, 1b cell height-1
; OVATT - byte 1 - overlay width and color shift
;         byte 2 - priority bits - bit0 - overlay above PM0
;                                  bit1 - ... PM1
;                                  bit2 - ... PM2
;                                  bit3 - ... PM3
;                                  bit4 - ... PF0
;                                  bit5 - ... PF1
;                                  bit6 - ... PF2
;                                  bit7 - ... PF3
; attributes map format
; byte 0 - colpf0 substitute color
; byte 1 - colpf1
; byte 2 - colpf2
; byte 3 - bits 0-1 - priority setting select
;          bit 2 - resolution override
;          bit 3 - reserved
;          bits 4-5 - overlay palette select
;          bits 6-7 - GTIA palette select
;
; Blitter Command Block
; bytes 0-2 source addr
; bytes 3-4 source line step
; bytes 5-7 dest. addr
; bytes 8-9 dest. line step
; bytes 10-11 obj width-1 in bytes
; byte 12 obj height
; byte 13 source AND mask
; byte 14 source XOR mask
; byte 15 collision AND mask
; byte 16 zoom bits 0-2 - x zoom
;              bit 3 - increment by 2 if set, else increment by 1
;              bits 4-6 - y zoom
; byte 17 pattern - bit 7 enables repeating source data horizontally
;                   bits 0-5 pattern width       
; byte 18 control - bits 0-2 - MODE
;                   bit3 - NEXT - 0 for last BCB
;                   bit4 - source x decrement if set, else increment
;                   bit5 - source y
;                   bit6 - dest. x
;                   bit7 - dest. y
; mode 0 - copy opaque
; mode 1 - copy with transparency
; mode 2 - source added to dest.
; mode 3 - OR
; mode 4 - AND
; mode 5 - XOR
; mode 6 - same as 1, with transparency and collision detection by nibbles
; mode 7 - reserved
VBXE_BASE			equ	0xd600		;d600, d700 or 0 (autodetection)
VBXE_VIDEO_CONTROL	equ	VBXE_BASE+0x40
VBXE_XDL_ADR0		equ	VBXE_BASE+0x41
VBXE_XDL_ADR1		equ	VBXE_BASE+0x42
VBXE_XDL_ADR2		equ	VBXE_BASE+0x43
VBXE_CSEL			equ	VBXE_BASE+0x44
VBXE_PSEL			equ	VBXE_BASE+0x45
VBXE_CR				equ	VBXE_BASE+0x46
VBXE_CG				equ	VBXE_BASE+0x47
VBXE_CB				equ	VBXE_BASE+0x48
VBXE_COLMASK		equ	VBXE_BASE+0x49
VBXE_COLCLR			equ	VBXE_BASE+0x4a
;VBXE_MA_CPU			equ	VBXE_BASE+0x4c
;VBXE_MA_ANTIC		equ	VBXE_BASE+0x4d
;VBXE_MB_CPU			equ	VBXE_BASE+0x4e
;VBXE_MB_ANTIC		equ	VBXE_BASE+0x4f
VBXE_BL_ADR0		equ	VBXE_BASE+0x50
VBXE_BL_ADR1		equ	VBXE_BASE+0x51
VBXE_BL_ADR2		equ	VBXE_BASE+0x52
VBXE_BLITTER_START	equ	VBXE_BASE+0x53
VBXE_BLITTER_BUSY	equ	VBXE_BASE+0x53
VBXE_IRQ_CONTROL	equ	VBXE_BASE+0x54

resx = 128
resy = 120
music_flag=0
stereomode=1

pointer equ $01 ;$02
pointer2 equ $03 ;$04
rotangle equ $05
sinlo	equ $06
sinhi	equ $07
coslo	equ $08
coshi	equ $09
frame 	equ $0a
cloc equ $14 ;$15
ulo		equ $16 ;ds 1
uhi		equ $17 ;ds 1
vlo		equ $18 ;ds 1
vhi		equ $19 ;ds 1
ustartlo	equ $1a ;ds 1
ustarthi	equ $1b ;ds 1
vstartlo	equ $1c ;ds 1
vstarthi	equ $1d ;ds 1
bank equ $0b
temp equ $0c

add_rh 	equ $ea00
add_gh	equ $eb00
add_bh	equ $ec00
add_rl 	equ $ed00
add_gl	equ $ee00
add_bl	equ $ef00

src_rh	equ $f000
src_gh	equ $f100
src_bh	equ $f200 
src_rl	equ $f300 
src_gl	equ $f400 
src_bl	equ $f500 
			
dest_rh	equ $f600 
dest_gh	equ $f700 
dest_bh	equ $f800 
dest_rl	equ $f900 
dest_gl	equ $fa00 
dest_bl	equ $fb00 



random	equ 53770

screen equ $40ba
startoffsetlo equ $7000 ;$000100 - $0001ff (128x2 lo,hi,lo,hi)
startoffsethi equ $7100 ;$000200
offsetlo equ $7200
offsethi equ $7300

sintable equ $7400 ;$000300
costable equ $7600 ;$000500


	
;	org $2000
;	lda #$84 ;$10000
;	sta $d65d
;	rts
;	ini $2000

;	org $4000
;	ins "text128x128.bmp",1078
	
;	org $2000
;	lda #$00 ;$7000
;	sta $d65d
;	rts
;	ini $2000

	org $2000
	lda #$80 
	sta $d65d
	lda #0
	sta 65
	sta 559
	sta $d400
	rts
	ini $2000

	org sintable ;$000300
	ins "rotosincostables.dat"
centersin	dta b(sin(95,255-96,128))
	dta b(sin(95,255-96,128))
	
	org $2000
	lda #$00 
	sta $d65d
	rts
	ini $2000
	
	org $2200	
logo_packed
title_packed
title1 		  ins    'text256x256.dat_001.cnk.LZ4',$0b,.FILESIZE 'text256x256.dat_001.cnk.lz4'-$0b-$06
title2 		  ins    'text256x256.dat_002.cnk.lz4',$0b,.FILESIZE 'text256x256.dat_002.cnk.lz4'-$0b-$06
title3 		  ins    'text256x256.dat_003.cnk.lz4',$0b,.FILESIZE 'text256x256.dat_003.cnk.lz4'-$0b-$06
title4		  ins    'text256x256.dat_004.cnk.lz4',$0b,.FILESIZE 'text256x256.dat_004.cnk.lz4'-$0b-$06
;title_pic0	ins 'desire_logo.dat_001.cnk.lz4',$0b,.FILESIZE 'desire_logo.dat_001.cnk.lz4'-$0b-$06
;title_pic1	ins 'desire_logo.dat_002.cnk.lz4',$0b,.FILESIZE 'desire_logo.dat_002.cnk.lz4'-$0b-$06
;title_pic2	ins 'desire_logo.dat_003.cnk.lz4',$0b,.FILESIZE 'desire_logo.dat_003.cnk.lz4'-$0b-$06
;title_pic3	ins 'desire_logo.dat_004.cnk.lz4',$0b,.FILESIZE 'desire_logo.dat_004.cnk.lz4'-$0b-$06
;title_pic4	ins 'desire_logo.dat_005.cnk.lz4',$0b,.FILESIZE 'desire_logo.dat_005.cnk.lz4'-$0b-$06

	org $1f00
.proc unlz4 
		  ldx #0
                  jsr    GET_BYTE                  ; length of literals
                  sta    token
                  lsr
                  lsr
                  lsr
                  lsr
                  beq    read_offset                     ; there is no literal
                  cmp    #$0f
                  jsr    getlength
literals          jsr    GET_BYTE
                  jsr    store
                  bne    literals
read_offset       jsr    GET_BYTE
                  tay
                  sec
                  eor    #$ff
                  adc    dest
                  sta    src
                  tya
                  php
                  jsr    GET_BYTE
                  plp
                  bne    not_done
                  tay
                  beq    unlz4_done
not_done          eor    #$ff
                  adc    dest+1
                  sta    src+1
                  ; c=1
                  lda    #$ff
token             equ    *-1
                  and    #$0f
                  adc    #$03                            ; 3+1=4
                  cmp    #$13
                  jsr    getLength

@                 lda    $ffff
src               equ    *-2
                  inc    src
                  bne    @+
                  inc    src+1
@                 jsr    store
                  bne    @-1
                  beq    unlz4                           ; zawsze
store             sta    $ffff
dest              equ    *-2
                  inc    dest
                  bne    @+
                  inc    dest+1
@                 dec    lenL
                  bne    @+
                  dec    lenH
@
unlz4_done        rts
getLength_next    jsr    GET_BYTE
                  tay
                  clc
                  adc    #$00
lenL              equ    *-1
                  bcc    @+
                  inc    lenH
@                 iny
getLength         sta    lenL
                  beq    getLength_next
                  tay
                  beq    @+
                  inc    lenH
@                 rts
lenH              .byte    $00
get_byte
		;lda #0
		stx $d65d		
	        lda  title_packed
source            equ    *-2
                  inw    source
		sta temp
		lda bank 
		sta $d65d
		lda temp
                  rts
;bank		.byte 0
unpacked	.word $4000
.endp


	org $2000
.proc do_unpack_texture
.local
	lda #<$4000
	sta unlz4.dest
	lda #>$4000
	sta unlz4.dest+1
	lda #<title1
	sta unlz4.source
	lda #>title1
	sta unlz4.source+1

	lda #$84
	sta $d65d
	sta bank
	jsr unlz4
	
	lda #<$4000
	sta unlz4.dest
	lda #>$4000
	sta unlz4.dest+1
	lda #<title2
	sta unlz4.source
	lda #>title2
	sta unlz4.source+1
	lda #$85 ;$018000 --> $4000 window
	sta bank
	jsr unlz4

	lda #<$4000
	sta unlz4.dest
	lda #>$4000
	sta unlz4.dest+1
	lda #<title3
	sta unlz4.source
	lda #>title3
	sta unlz4.source+1
	lda #$86 ;$01c000 --> $4000 window
	sta bank
	jsr unlz4

	lda #<$4000
	sta unlz4.dest
	lda #>$4000
	sta unlz4.dest+1
	lda #<title4
	sta unlz4.source
	lda #>title4
	sta unlz4.source+1
	lda #$87 ;$01c000 --> $4000 window
	sta bank
	jsr unlz4	
	lda #$13
	sta $7ffc
	sta $7ffd
	sta $7ffe
	sta $7fff
;	lda #<$4000
;	sta unlz4.dest
;	lda #>$4000
;	sta unlz4.dest+1
;	lda #<title_pic0
;	sta unlz4.source
;	lda #>title_pic0
;	sta unlz4.source+1
;	lda #$98
;	sta $d65d
;	sta bank
;	jsr unlz4
;	lda #<$4000
;	sta unlz4.dest
;	lda #>$4000
;	sta unlz4.dest+1
;	lda #<title_pic1
;	sta unlz4.source
;	lda #>title_pic1
;	sta unlz4.source+1
;	lda #$99
;	sta $d65d
;	sta bank
;	jsr unlz4
;	lda #<$4000
;	sta unlz4.dest
;	lda #>$4000
;	sta unlz4.dest+1
;	lda #<title_pic2
;	sta unlz4.source
;	lda #>title_pic2
;	sta unlz4.source+1
;	lda #$9a
;	sta $d65d
;	sta bank
;	jsr unlz4
;	lda #<$4000
;	sta unlz4.dest
;	lda #>$4000
;	sta unlz4.dest+1
;	lda #<title_pic3
;	sta unlz4.source
;	lda #>title_pic3
;	sta unlz4.source+1
;	lda #$9b
;	sta $d65d
;	sta bank
;	jsr unlz4
	rts		
.endl
.endp 
	ini do_unpack_texture


	org $2000
	lda #$00 
	sta $d65d
	rts
	ini $2000
	
	org $2200
desire_packed	
title_pic0	ins 'desire_logo.dat_001.cnk.lz4',$0b,.FILESIZE 'desire_logo.dat_001.cnk.lz4'-$0b-$06
title_pic1	ins 'desire_logo.dat_002.cnk.lz4',$0b,.FILESIZE 'desire_logo.dat_002.cnk.lz4'-$0b-$06
title_pic2	ins 'desire_logo.dat_003.cnk.lz4',$0b,.FILESIZE 'desire_logo.dat_003.cnk.lz4'-$0b-$06
title_pic3	ins 'desire_logo.dat_004.cnk.lz4',$0b,.FILESIZE 'desire_logo.dat_004.cnk.lz4'-$0b-$06
title_pic4	ins 'desire_logo.dat_005.cnk.lz4',$0b,.FILESIZE 'desire_logo.dat_005.cnk.lz4'-$0b-$06

	org $2000
.proc do_unpack_title
.local
	lda #<$4000
	sta unlz4.dest
	lda #>$4000
	sta unlz4.dest+1
	lda #<title_pic0
	sta unlz4.source
	lda #>title_pic0
	sta unlz4.source+1
	lda #$98
	sta $d65d
	sta bank
	jsr unlz4
	lda #<$4000
	sta unlz4.dest
	lda #>$4000
	sta unlz4.dest+1
	lda #<title_pic1
	sta unlz4.source
	lda #>title_pic1
	sta unlz4.source+1
	lda #$99
	sta $d65d
	sta bank
	jsr unlz4
	lda #<$4000
	sta unlz4.dest
	lda #>$4000
	sta unlz4.dest+1
	lda #<title_pic2
	sta unlz4.source
	lda #>title_pic2
	sta unlz4.source+1
	lda #$9a
	sta $d65d
	sta bank
	jsr unlz4
	lda #<$4000
	sta unlz4.dest
	lda #>$4000
	sta unlz4.dest+1
	lda #<title_pic3
	sta unlz4.source
	lda #>title_pic3
	sta unlz4.source+1
	lda #$9b
	sta $d65d
	sta bank
	jsr unlz4
	lda #<$4000
	sta unlz4.dest
	lda #>$4000
	sta unlz4.dest+1	
	lda #<title_pic4
	sta unlz4.source
	lda #>title_pic4
	sta unlz4.source+1
	lda #$9c
	sta $d65d
	sta bank
	jsr unlz4
	rts	
.endl
.endp 

	ini do_unpack_title

	
;main
	org $1000
.proc	NMI
	bit $d40f
	bpl VBL
	jmp dli
dliv	equ *-2

VBL
	sta vbl_a+1
	stx vbl_x+1
	sty vbl_y+1
;	inc rotangle
;	inc frame
	
	dec cloc
	bne @+
	dec cloc+1
	lda cartbank+1
	sta vbl0+1
@	
	sta $d500+22 ;music in bank #22
	jsr rmtplayer+3
vbl0	sta $d500
	
	lda scrollstartflag
	beq vbl99


	lda cloc
	and #2
	beq @+

	inc blinkcolor
	lda blinkcolor
	and #$0f
	ora #$f0
	sta $d017

	inc svscrol
	lda svscrol
	sta $d405
	cmp #16
	bne @+
;scroll
	lda #0
	sta svscrol
	lda textad
	clc
	adc #16
	sta textad
	lda textad+1
	adc #0
	sta textad+1
	cmp #>textend
	bne @+
	lda textad
	cmp #<textend
	bne @+
	lda #<text
	sta textad
	lda #>text
	sta textad+1	
@

vbl99
vbl_a	lda #0
vbl_x	ldx #0
vbl_y	ldy #0
dli	rti

.endp
memwindow .byte 0
memwindow2 .byte 0
svscrol	.byte 0
scrollstartflag .byte 0
blinkcolor .byte $00

dlist
	.byte $50,$67
textad	.word text
:13	.byte $27
	.byte $07
	.byte $41
	.word dlist


; init
init

	ldx 53770
@
	dex
	bpl @-
	
	lda #0
	ldx #<msx
	ldy #>msx
	jsr rmtplayer
	
	
        lda #$80		;bank 0
        sta $d65d       ;cpu-vram access window at $4000
	ldy #5
        ldx #0
a3:     lda xdl,x
a33        sta $6000,x     ; copy XDL to vram
        inx
        bne a3
	inc a3+2
	inc a33+2
	dey
	bne a3

	ldy #5
        ldx #0
a30:     lda xdl2,x
a330        sta $6500,x     ; copy XDL to vram
        inx
        bne a30
	inc a30+2
	inc a330+2
	dey
	bne a30

	ldx #0
@	lda copy_bcb,x
	sta $4020,x
	lda clear_bcb,x
	sta $40c0,x
	inx
	cpx #21
	bne @-
	
	ldx #0
@	lda blitgen_bcb,x
	sta $4040,x
	inx
	cpx #21
	bne @-

	ldx #0
@	lda pixel_bcb,x
	sta $4060,x
	inx
	cpx #21
	bne @-

	ldx #0
@	lda moveline_bcb,x
	sta $4080,x
	inx
	cpx #63
	bne @-
;setting uv
@	
	ldx #0
qwq	lda setstartfx_bcb,x
qwq2	sta $4100,x
	inx
	cpx #4*42
	bne qwq
	
	lda #$40
	sta $d650
	lda #$00
	sta $d651
	sta $d652
		
;generate 128 blits (1 line)	
@
	ldx #resx	
@
	lda #1
	sta $d653
	jsr waitblt
	
	lda $4046
	clc
	adc #21
	sta $4046
	lda $4047
	adc #0
	sta $4047
	lda $4048
	adc #0
	sta $4048
		
	inc $4060
	inc $4066

	dex
	bne @-

	lda #0
	sta $4060+20
	lda #1
	sta $d653
	jsr waitblt
		

init_end
	lda #$00
	sta $d65d
	tax
@
	lda font,x
	sta $b000,x
	lda font+$100,x
	sta $b100,x
	lda font+$200,x
	sta $b200,x
	lda font+$300,x
	sta $b300,x
	inx
	bne @-
;	
@		lda $d40b
		bne @-
		sei			;stop IRQ interrupts
		mva #$00 $d40e		;stop NMI interrupts	
		sta $d400
		mva #$fe $d301		;switch off ROM to get 16k more ram

		lda #<nmi
		sta $fffa
		sta $fffe
		lda #>nmi
		sta $fffb
		sta $ffff
		lda #$20
		sta $d650
		lda #0
		sta $d651
		sta $d652
		lda #1
		sta $d653
		jsr waitblt
		jsr precalc_uv
		cli
		lda #$40
		sta $d40e
		lda #<dlist
		sta $d402
		lda #>dlist
		sta $d403
		lda #15
		sta $d016
		lda #33
		sta $d400
		lda #$b0
		sta $d409
	lda #0
        sta $D641       ;
        lda #$20
        sta $D642       ;
        lda #$00          ; XDL addr 
        sta $D643       ;
        lda #0
        sta $D647       ; disable collision detection
        lda #0
        sta $D654       ; disable IRQ
        lda #1
        sta $D640       ; enable XDL
	
	lda #0
	sta rotangle
	sta frame
	lda #7
	sta screen

	jmp exit_fx
	
main_demo_loop
loop
xdlflag lda #$25
        sta $D642       ;
	eor #5
	sta xdlflag+1
	
@	lda cloc
	cmp cloc
	beq *-2
	jsr do_copy_tables
	jsr render_scene
	
	lda exitflag
	cmp #3
	beq @+
	
	lda #2
	sta cloc
	jmp loop
@
	jmp loop ;exit_fx
render_scene
	ldx #resy
@	
;copy 1 line uv into line blit
	lda #$00
	sta $d650
	lda #$01
	sta $d651
	lda #0
	sta $d652		
	lda #1
	sta $d653
	jsr waitblt

;blit one line 128 pixels on screen
	lda #$00
	sta $d650
	lda #$00
	sta $d651
	lda #$02
	sta $d652
	lda #1
	sta $d653
	jsr waitblt

	inc $4100+21*2 ;line startoffsets u table++
	inc $4100+21*3 ;line startoffsets v table++
;move one line	
	lda #$80
	sta $d650
	lda #$00
	sta $d651
	lda #$00
	sta $d652
	lda #1
	sta $d653
	jsr waitblt
	dex
	bne @-	
;reset lines
	lda screen
	eor #2
	sta screen	

	lda #$80+21
	sta $d650
	lda #$00
	sta $d651
	lda #$00
	sta $d652
	lda #1
	sta $d653
	jsr waitblt
	;brk
	lda #0
	sta $4100+21*2 ;line startoffsets u table++
	sta $4100+21*3 ;line startoffsets v table++	
	rts

waitblt:
        lda $D653       ; wait until not-busy
        bne waitblt
        rts

.proc precalc_uv
		lda #0
		sta rotangle
		sta frame
		
rotozoom
		lda #$80
		sta $d65d
		ldx rotangle
		lda sintable,x
		sta sinlo
		lda sintable+256,x
		sta sinhi
		lda costable,x
		sta coslo
		lda costable+256,x
		sta coshi
		
		ldx frame
		lda centersin,x
		asl
		clc
		adc frame
		tay
		sty vstarthi
		sty inity+1
		
		ldx #0 ;frame		
		stx ustarthi
		stx initx+1
;prepare y-offsets
;For a=0 To 101
;  startoffset(a)=(fx_start>>8)&$3f+((fy_start>>8)&$3f)*128
;  fx_start.u=fx_start.u+cos.u
;  fy_start.u=fy_start.u+sin.u
;;Next a
;[07.08.2016 21:45:03] Oswald: to rotate it in the middle
;[07.08.2016 21:45:13] Oswald: you start the offsettabs in the middle
;[07.08.2016 21:45:21] Oswald: and in one dir you adc, other dir sbc
;[07.08.2016 21:45:28] Oswald: just the opposite always
;		ldx #22

		tya

rotozoom0	
.rept 64 
		sta startoffsetlo+[63-#]
		txa
		sec
		sbc coslo
		tax
		tya
		sbc coshi
		tay
.endr
		ldx #0
		ldy #0
		tya
.rept 64 ;22		
		sta startoffsethi+[63-#] ;,x
		txa
		sec
		sbc sinlo
		tax
		tya
		sbc sinhi
		tay				
.endr		
initx		ldx #0
		stx ustarthi
inity		ldy #0		
		sty vstarthi		
;		ldx #23

rotozoom00	
.rept 64 ;23
		txa
		clc
		adc coslo
		tax

		tya
		adc coshi
		tay
		sta startoffsetlo+64+# ;,x

.endr
		lda #0
		tax
		tay
.rept 64 ;23
		txa
		clc
		adc sinlo
		tax
		tya
		adc sinhi
		tay	
		sta startoffsethi+64+# ;,x
.endr		
;		inx
;		cpx #46
;		bne rotozoom00
		
;fx.u=0
;fy.u=0
;  For a=0 To 159
;    offset.u(a)=(fx.u>>8)&$3f+((fy>>8)&$3f)*128
;    fx.u=fx.u+sin.u
;    fy.u=fy.u-cos.u
;  Next a
		lda #0
		sta uhi
		sta vlo
		sta vhi
		tay
		tax	
;		ldx #21
.rept 64 ;22
;rotozoom1	
		txa
		sec
		sbc sinlo
		tax
		tya
		sbc sinhi
		tay
		sta offsetlo+[63-#] ;,x
.endr
		ldx #0
		ldy #0
.rept 64 ;22
		txa
		clc
		adc coslo
		tax
		tya
		adc coshi
		tay
		sta offsethi+[63-#] ;,x
.endr 
		lda #0
		sta uhi
		sta vlo
		sta vhi
		tay
		tax

;		ldx #22
.rept 64 ;23
;rotozoom2	
		sta offsetlo+64+# ;,x
		txa
		clc
		adc sinlo
		tax
		tya
		adc sinhi
		tay
.endr
		lda #0
		tax
		tay
.rept 64 ;23
		;lda vlo
;		lda vhi
;		and #$7f
;		ora #$40
		sta offsethi+64+# ;,x

		txa
		sec
		sbc coslo
		tax
		;sta vlo
		;lda vhi
		tya
		sbc coshi
		;sta vhi
		tay
.endr
		
;copy tables to desired place
;startoffsetlo equ $7000 ;$000100 - $0001ff (128x2 lo,hi,lo,hi)
;startoffsethi equ $7100 ;$000200
;offsetlo equ $7200
;offsethi equ $7300
;copy 512 bytes
		lda #0
		sta $4020
		sta $4022
src		lda #$30
		sta $4021
dstlo		lda #$00
		sta $4026
dstmid		lda #$00
		sta $4027
dsthi		lda #$03
		sta $4028
		lda #$20
		sta $d650
		lda #$00
		sta $d651
		sta $d652
		lda #1 
		sta $d653
		jsr waitblt
		lda dstmid+1
		clc
		adc #2
		sta dstmid+1
		lda dsthi+1
		adc #0
		sta dsthi+1
		inc frame
		inc rotangle
;		inc rotangle
		beq @+
		jmp rotozoom
@
;		brk
		lda #128
		sta $4023
		lda #0
		sta $4024
		sta $4029
		lda #1
		sta $402a	
		rts
.endp

.proc do_copy_tables
src		lda #$30 ;$3000 - $337f
		sta $4027
		lda #0
		sta $4026
		sta $4028		
dstlo		lda #$00
		sta $4020
dstmid		lda #$00
		sta $4021
dsthi		lda #$03
		sta $4022
		lda #$20
		sta $d650
		lda #$00
		sta $d651
		sta $d652
		lda #1 
		sta $d653
		jsr waitblt
		
		lda dstmid+1
		clc
		adc #2
		sta dstmid+1
		lda dsthi+1
		adc #0
		sta dsthi+1
		cmp #$05
		bne @+
		lda #$03
		sta dsthi+1
		inc exitflag
@	
		rts
.endp
exitflag	.byte 0

	.align $100
:$3c0	.byte 0
		.align $100
rmtplayer	icl "rmtplayr_triace.asm"

;--> $2b00 = $6b00
xdl:
        .byte $72 ;repeat+adr+gfx
        .byte $28 
        .byte 1     ;RPTL
;screenram
vramadr	.long $070000 ;OVADR 
	.byte <0,>0  ;ystep of display          
        .byte $10,$c0   ;OVATT
.rept 118
        .byte $72 ;repeat+adr+gfx
        .byte $28 
        .byte 1     ;RPTL
	.long $070100+#*256 ;OVADR 
	.byte 0,0  ;ystep of display          
        .byte $10,$c0   ;OVATT
.endr
        .byte $72 ;repeat+adr+gfx
        .byte $a8 
        .byte 1     ;RPTL
	.long $070000+119*256 ;OVADR 
	.byte <0,>0  ;ystep of display          
        .byte $10,$c0   ;OVATT

xdl2:
        .byte $72 ;repeat+adr+gfx
        .byte $28 
        .byte 1     ;RPTL
;screenram
vramadr2 .long $050000 ;OVADR 
	.byte <0,>0  ;ystep of display          
        .byte $10,$c0   ;OVATT
.rept 118
        .byte $72 ;repeat+adr+gfx
        .byte $28 
        .byte 1     ;RPTL
	.long $050100+#*256 ;OVADR 
	.byte 0,0  ;ystep of display          
        .byte $10,$c0   ;OVATT
.endr
        .byte $72 ;repeat+adr+gfx
        .byte $a8 
        .byte 1     ;RPTL
	.long $050000+119*256 ;OVADR 
	.byte <0,>0  ;ystep of display          
        .byte $10,$c0   ;OVATT

xdl3:
        .byte $72 ;repeat+adr+gfx
        .byte $88 
        .byte 239     ;RPTL
;screenram
vramadr3 .long $060000 ;OVADR 
	.byte <320,>320  ;ystep of display          
        .byte $11,$c0   ;OVATT

clear_bcb
		.long $000000 ;source adress ;$0400 texture $005000
		.word 0 ;source step y
		.byte 0 ;source step x
		.long $050000 ;destination adress
		.word 256 ;dest. step y
		.byte 1
		.word 255 ;size x
		.byte 239   ;size y
		.byte $00 	;and
	        .byte $00     ; XOR
	        .byte 0       ;  collision AND
 	        .byte $00       ; zoom
		.byte $00       ; pattern
	        .byte 0; control

copy_bcb
		.long $003000 ;source adress ;$0400 texture $005000
		.word 256 ;source step y
		.byte 1 ;source step x
		.long $030000 ;destination adress
		.word 128 ;dest. step y
		.byte 1
		.word 127 ;size x
		.byte 3   ;size y
		.byte $ff 	;and
	        .byte $00     ; XOR
	        .byte 0       ;  collision AND
 	        .byte $00       ; zoom
		.byte $00       ; pattern
	        .byte 0; control
pixel_bcb
		.long $010000 ;source adress ;$0400 texture $005000
		.word 0 ;source step y
		.byte 0 ;source step x
		.long $070000 ;destination adress
		.word 0 ;dest. step y
		.byte 0
		.word 0 ;size x
		.byte 0	;size y
		.byte $ff 	;and
	        .byte $00     ; XOR
	        .byte 0       ;  collision AND
 	        .byte $00       ; zoom
		.byte $00       ; pattern
	        .byte 8; control

moveline_bcb
		.long $000000 ;source adress ;$0400 texture $005000
		.word 0 ;source step y
		.byte 21 ;source step x
		.long $020007 ;destination adress
		.word 0 ;dest. step y
		.byte 21
		.word 127 ;size x
		.byte 0	;size y
		.byte $00 	;and
	        .byte $01     ; XOR
	        .byte 0       ;  collision AND
 	        .byte $00       ; zoom
		.byte $00       ; pattern
	        .byte 2; control
resetline_bcb
		.long $000000 ;source adress ;$0400 texture $005000
		.word 0 ;source step y
		.byte 21 ;source step x
		.long $020007 ;destination adress
		.word 0 ;dest. step y
		.byte 21
		.word 127 ;size x
		.byte 0	;size y
		.byte $00 	;and
	        .byte $00     ; XOR
	        .byte 0       ;  collision AND
 	        .byte $00       ; zoom
		.byte $00       ; pattern
	        .byte 8; control
;double buffer
		.long $000000 ;source adress ;$0400 texture $005000
		.word 0 ;source step y
		.byte 0 ;source step x
		.long $020008 ;destination adress
		.word 0 ;dest. step y
		.byte 21
		.word 127 ;size x
		.byte 0	;size y
		.byte $00 	;and
	        .byte $07     ; XOR
	        .byte 0       ;  collision AND
 	        .byte $00       ; zoom
		.byte $00       ; pattern
	        .byte 0; control
;
blitgen_bcb
		.long $000060 ;source adress ;$0400 texture $005000
		.word 0 ;source step y
		.byte 1 ;source step x
		.long $020000 ;destination adress
		.word 0 ;dest. step y
		.byte 1
		.word 20 ;size x
		.byte 0	;size y
		.byte $ff 	;and
	        .byte $00     ; XOR
	        .byte 0       ;  collision AND
 	        .byte $00       ; zoom
		.byte $00       ; pattern
	        .byte 0; control

;		org $8000
		.align $100
setstartfx_bcb ;copies fx_start into startoffset table
.rept 1
		.long $003200 ;source adress ;$0400 texture $005000
		.word 0 ;source step y
		.byte 1 ;source step x
		.long $020000 ;destination adress
		.word 0 ;dest. step y
		.byte 21
		.word 127 ;size x
		.byte 0 ;127	;size y
		.byte $ff 	;and
	        .byte $00     ; XOR
	        .byte 0       ;  collision AND
 	        .byte $00       ; zoom
		.byte $00       ; pattern
	        .byte 8; control

		.long $003300 ;source adress ;$0400 texture $005000
		.word 0 ;source step y
		.byte 1 ;source step x
		.long $020001 ;destination adress
		.word 0 ;dest. step y
		.byte 21
		.word 127 ;size x
		.byte 0 ;size y
		.byte $ff 	;and
	        .byte $00     ; XOR
	        .byte 0       ;  collision AND
 	        .byte $00       ; zoom
		.byte $00       ; pattern
	        .byte 8; control
;add startoffset
		.long $003000 ;source adress ;$0400 texture $005000
		.word 0 ;source step y
		.byte 0 ;source step x
		.long $020000 ;destination adress
		.word 0 ;dest. step y
		.byte 21
		.word 127 ;size x
		.byte 0 ;127	;size y
		.byte $ff 	;and
	        .byte $00     ; XOR
	        .byte 0       ;  collision AND
 	        .byte $00       ; zoom
		.byte $00       ; pattern
	        .byte 10; control

		.long $003100 ;source adress ;$0400 texture $005000
		.word 0 ;source step y
		.byte 0 ;source step x
		.long $020001 ;destination adress
		.word 0 ;dest. step y
		.byte 21
		.word 127 ;size x
		.byte 0 ;size y
		.byte $ff 	;and
	        .byte $00     ; XOR
	        .byte 0       ;  collision AND
 	        .byte $00       ; zoom
		.byte $00       ; pattern
	        .byte 2; control

.endr


		org $8000
title_pal	ins "desire_logo.bmp",54,1024
	
exit_fx

	lda #$80
	sta $d65d

	ldx #9
@
	lda xdl3,x
	sta $4000,x
	dex
	bpl @-
.local
VBXE_SetPalette:

		ldx	#1
		stx	VBXE_PSEL
		ldx	#0 ;108
		stx	VBXE_CSEL ;which colour # to change
		ldy #0
setcolloop	
setcol		lda title_pal+2
		;tya
		;sta VBXE_CR
		lsr
		sta dest_rh,y
setcol1		lda title_pal+1
		;tya
		;sta VBXE_CG
		lsr
		sta dest_gh,y
setcol2		lda title_pal
		;tya
		;sta VBXE_CB
		lsr
		sta dest_bh,y
		lda setcol+1
		clc
		adc #4
		sta setcol+1
		lda setcol+2
		adc #0
		sta setcol+2
		lda setcol1+1
		clc
		adc #4
		sta setcol1+1
		lda setcol1+2
		adc #0
		sta setcol1+2
		lda setcol2+1
		clc
		adc #4
		sta setcol2+1
		lda setcol2+2
		adc #0
		sta setcol2+2
		iny
		bne setcolloop
.endl

;palette to black
	ldx #0
	txa
@	stx VBXE_CSEL
	sta VBXE_CR
	sta VBXE_CG
	sta VBXE_CB
	sta src_rh,x
	sta src_gh,x
	sta src_bh,x
	sta src_rl,x
	sta src_gl,x
	sta src_bl,x
	inx
	bne @-
	
	clc
	jsr calc_fade_steps

@
	lda cloc+1
	bne @-
	lda #0
	sta $d400	

	lda #0
        sta $D641       ;
        lda #$00
        sta $D642       ;
        lda #$00          ; XDL addr 
        sta $D643       ;
        lda #0
        sta $D647       ; disable collision detection
        lda #0
        sta $D654       ; disable IRQ
        lda #1
        sta $D640       ; enable XDL

	lda #125
	sta cloc
	lda cloc
	bne *-2

	lda #250
	sta cloc
	lda cloc
	bne *-2

	lda #32
	sta fadecounter

;fade in
@
	lda cloc
	cmp cloc
	beq *-2
	jsr fade_to_palette
	dec fadecounter
	bne @-	

	lda #200
	sta cloc
	lda cloc
	bne *-2
	
	tax
@	sta dest_rl,x
	sta dest_gl,x
	sta dest_bl,x
	sta dest_rh,x
	sta dest_gh,x
	sta dest_bh,x
	inx
	bne @-

	sec
	jsr calc_fade_steps
	
	lda #32
	sta fadecounter
		
;fade out
@
	lda cloc
	cmp cloc
	beq *-2
	jsr fade_to_palette
	dec fadecounter
	bne @-	

;palette to black
	ldx #0
	txa
@	stx VBXE_CSEL
	sta VBXE_CR
	sta VBXE_CG
	sta VBXE_CB
	inx
	bne @-

	lda #$c0
	sta $d650
	lda #0
	sta $d651
	sta $d652
	lda #1
	sta $d653
	jsr waitblt
	
	lda #0
	tax
	sta VBXE_CSEL
@	
	stx VBXE_CR
	stx VBXE_CG
	stx VBXE_CB
	inx
	bne @-
	
	jsr VBXE_SetPalette

	
	lda #1
	sta cloc
	lda cloc
	bne *-2

	
	lda #33
	sta $d400
	sta scrollstartflag
	lda #$b0
	sta $d01a
	lda #$ff
	sta $d00d
	sta $d00e
	sta $d00f
	sta $d010
	lda #$00
	sta $d012
	sta $d013
	sta $d014
	sta $d015
	lda #1
	sta 53256
	sta 53257
	sta 53258
	sta 53259
	lda #$30
	sta 53248
	lda #$20
	sta 53249
	lda #$c0
	sta 53250
	lda #$d0
	sta 53251
	lda #1
	sta $d01b
	jmp main_demo_loop	
partexit	
	lda #0
	sta $d640
	jsr rmt_silence
	sei
	lda #0
	sta $d40e
	
	lda #0
	sta $d65d ;switch off vbxe banks
	lda #23+5+8 ;gouraud part
	sta cartbank+1
	lda #$0c
	sta ad4+2
	jsr move_block
	jmp $1664




	sta $d500+23
	sta $d65d
	sta $d65f
	jmp $a000 ;start fire	
	
	

;copies 16k to $4000

move_block lda #1
mm	ldy #32
	ldx #0
cartbank sta $d500
ad3	lda $a000,x
;	lda #$11
ad4	sta $4000,x
	inx
	bne cartbank
	inc ad3+2
	inc ad4+2
	dey
	bne cartbank ;8k copied 
	inc cartbank+1 ;next bank	
	lda #$a0
	sta ad3+2
	dec move_block+1
	bpl mm
	lda #1
	sta move_block+1
	lda #$40
	sta ad4+2
	rts	


VBXE_SetPalette:

		ldx	#1
		stx	VBXE_PSEL
		ldx	#0 ;108
		stx	VBXE_CSEL ;which colour # to change
		ldy #0
setcolloop	
setcol		lda pal1+2
		;tya
		sta VBXE_CR
setcol1		lda pal1+1
		;tya
		sta VBXE_CG
setcol2		lda pal1
		;tya
		sta VBXE_CB
		lda setcol+1
		clc
		adc #4
		sta setcol+1
		lda setcol+2
		adc #0
		sta setcol+2
		lda setcol1+1
		clc
		adc #4
		sta setcol1+1
		lda setcol1+2
		adc #0
		sta setcol1+2
		lda setcol2+1
		clc
		adc #4
		sta setcol2+1
		lda setcol2+2
		adc #0
		sta setcol2+2
		iny
		bne setcolloop
		lda #0
		sta VBXE_CSEL
		lda #$FF
		sta VBXE_CR
		sta VBXE_CG
		sta VBXE_CB
		rts
		.align $100
		
pal1		ins "text256x256.bmp",54,1024
font		ins "antic.chr"

	org $a000
text
:37	.byte d"                "
	.byte d" DESIRE INVITES "
	.byte d"     YOU TO     "
	.byte d"                "
	.byte d"   COMPUSPHERE  "
	.byte d"   -----------  "
	.byte d"   2222  55555  " ; <- BLINK
	.byte d"      22 55     " ; <- BLINK
	.byte d"    222  5555   " ; <- BLINK
	.byte d"   22       55  " ; <- BLINK
	.byte d"   22222 5555   " ; <- BLINK
	.byte d"   -----------  " 
	.byte d"                "
	.byte d"                "
	.byte d" 30 NOV - 2 DEC "
	.byte d"   GOTHENBURG   "
	.byte d"     SWEDEN     "
	.byte d"                "
	.byte d"                "
	.byte d"  COMPUSPHERE   "
	.byte d"                "
	.byte d"stronger        " ; <- BLINK
	.byte d"    better      " ; <- BLINK
	.byte d"        forever " ; <- BLINK
	.byte d"                "
	.byte d"                "
	.byte d"    25 YEAR     " ;<- 2 empty spaces between 25 and Years to get fully centered
	.byte d"- ANNIVERSARY - "
	.byte d"                "
	.byte d"                "
	.byte d"                "
	.byte d"  2018 EDITION  "
	.byte d"   BRINGS YOU:  "
	.byte d"                " ;<- add empty row
	.byte d"CENTRAL LOCATION"
	.byte d"   COZY VENUE   "
	.byte d"OLDSKOOL  COMPOS"
	.byte d"TECH-TECH TALKS "
	.byte d"SHADER GRANDPRIX"
	.byte d"  MIDNIGHT BBQ  "
	.byte d"   LIVE  ACTS   "
	.byte d" LIVE BROADCAST "
	.byte d"                "
	.byte d"                " ;<- add empty row x 2
	.byte d"                "
	.byte d" remote entries " ;<- BLINK
	.byte d"    ACCEPTED    " ;<- BLINK (add one space in front for fully centered 
	.byte d"                "
	.byte d"                "
	.byte d"                "
	.byte d"  REGISTER NOW  " ;<- remove one space before to get fully centered
	.byte d" COMPUSPHERE.SE "
	.byte d"                "
	.byte d"                " ;<- add empty row
	.byte d" COMPUSPHERE:25 " ;<- add one space in front for fully centered
	.byte d" WE LOVE ATARI! "
	.byte d" SEE YOU THERE! "
	.byte d"                "
	.byte d"                "
	.byte d"      ---       "
	.byte d"                "
	.byte d"                "
	.byte d"THIS SMALL PROD."
	.byte d"  RELEASED AT   "
	.byte d"                " ;<- add empty row
	.byte d" SILLY VENTURE  "
	.byte d"     2018       "
	.byte d"                "
	.byte d"                "
	.byte d"CODE:     HEAVEN"
	.byte d"                "
	.byte d"MUSIC:    TRIACE"
	.byte d"                "
	.byte d"GFX:       ALIEN"
	.byte d"          (LOGO)"
	.byte d"                "
	.byte d"            EROS"
	.byte d"       (TEXTURE)"
	.byte d"                "
	.byte d"TEXT:      COREL"
	.byte d"                "
	.byte d"                "
	.byte d"      ---       "
	.byte d"                "
	.byte d"                "
	.byte d" 8-BIT LOVE TO: "
	.byte d"                "
	.byte d"          AGENDA" ;<- Don´t know if you have an official Desire greetings list, but used this one from Gnork! aas a starter
	.byte d"           AVENA"
	.byte d"         ARSENIC"
	.byte d"        ATTITUDE"
	.byte d"          AVATAR"
	.byte d"      BRAINSTORM"
	.byte d"         COSINE "
	.byte d"        DARKLITE"
	.byte d"       DEKADENCE"
	.byte d"     DREADLINERS"
	.byte d"        GHOSTOWN"
	.byte d" GENESIS PROJECT"
	.byte d"    HOKUTO FORCE"
	.byte d"          INSANE"
	.byte d"        KREWLERS"
	.byte d"          LAMERS"
	.byte d"MANIACS OF NOICE"
	.byte d"       NAH KOLOR"
	.byte d"          NUANCE"
	.byte d"       OCS FARTS"
	.byte d"         OFFENCE"
	.byte d"       ONSLOUGHT"
	.byte d"         OXYGENE"
	.byte d"          OXYRON"
	.byte d"       POO BRAIN"
	.byte d"         QUARTEX"
	.byte d"          REBELS"
	.byte d"      RESISTANCE"
	.byte d"        RESOURCE"
	.byte d"             TBL"
	.byte d"           TITAN"
	.byte d"        TRISTESE"
	.byte d"            TRSI"
	.byte d"            TSCC"
	.byte d"                "
	.byte d"                "
	.byte d"                "
	.byte d"    SUPPORT     "
	.byte d"  COMPUSPHERE   "
	.byte d"     WE DO      "
	.byte d"                "
	.byte d"  DESIRE 2018   "
	.byte d"                "
	.byte d"                "
	.byte d"      ---       "
	.byte d"                "
	.byte d"                "
	.byte d"  EOL.RESTART.  "
	.byte d"   SCROLLER.    "

textend
:14	.byte d"                "

calc_fade_steps
	ldx #$00
	txa 
fade00	
;	lda #0
	sta add_rl,x
	sta add_bl,x
	sta add_gl,x
	sta add_rh,x
	sta add_bh,x
	sta add_gh,x
	bcs @+
	sta src_rl,x
	sta src_gl,x
	sta src_bl,x
	sta src_rh,x
	sta src_gh,x
	sta src_bh,x	
@
	inx
	bne fade00
;calc steps
	ldx #$00
calc_fade0 lda dest_gl,x
	sec
	sbc src_gl,x 
	sta add_gl,x
	lda dest_gh,x
	sbc src_gh,x
c0	cmp #$80
	ror
	ror add_gl,x
	cmp #$80
	ror
	ror add_gl,x
	cmp #$80
	ror
	ror add_gl,x
	cmp #$80
	ror
	ror add_gl,x	
	sta add_gh,x
	inx
	bne calc_fade0
	
calc_fade1 lda dest_bl,x
	sec
	sbc src_bl,x
	sta add_bl,x
	lda dest_bh,x
	sbc src_bh,x
	cmp #$80
	ror
	ror add_bl,x
	cmp #$80
	ror
	ror add_bl,x
	cmp #$80
	ror
	ror add_bl,x
	cmp #$80
	ror
	ror add_bl,x
	sta add_bh,x
	inx
	bne calc_fade1
calc_fade2 lda dest_rl,x
	sec
	sbc src_rl,x
	sta add_rl,x
	lda dest_rh,x
	sbc src_rh,x
	cmp #$80
	ror
	ror add_rl,x
	cmp #$80
	ror
	ror add_rl,x
	cmp #$80
	ror
	ror add_rl,x
	cmp #$80
	ror
	ror add_rl,x
	sta add_rh,x
	inx
	bne calc_fade2
	rts

fade_counter .byte 15
fade_delay .byte 2

fade_to_palette
	dec fade_delay
	beq fadeloop
	rts
;	jsr calc_fade_steps
	
;	ldx #16
fadeloop
	lda #2
	sta fade_delay	
	ldy #$00
	sty vbxe_csel
fade4 	lda src_rh,y
	asl
	sta VBXE_CR
	lda src_gh,y
	asl
	sta VBXE_CG
	lda src_bh,y
	asl
	sta VBXE_CB
	
	lda src_rl,y
	clc
	adc add_rl,y
	sta src_rl,y
	lda src_rh,y
	adc add_rh,y
	sta src_rh,y
	
	lda src_gl,y
	clc
	adc add_gl,y
	sta src_gl,y
	lda src_gh,y
	adc add_gh,y
	sta src_gh,y
	
	lda src_bl,y
	clc
	adc add_bl,y
	sta src_bl,y
	lda src_bh,y
	adc add_bh,y
	sta src_bh,y
	iny
	bne fade4
fadein
	rts
;g,b,r
fadecounter .byte 32

	org $b800
msx	ins "sonariusv1.rmt",6
	run init
	
